import click


@click.group()
def cli() -> None:
    """i8 Terminal - Modern Market Research with the Power of Terminal

    Copyright © 2020-2022 Investoreight | https://www.investoreight.com/"""
    pass
